// Copyright 2013 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package bug

type S struct {
	F func()
}

type X interface {
	Bar()
}

func Foo(x X) *S {
	return &S{F: x.Bar}
}
