// Copyright 2019 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package main

import "./a"

type No struct {
	a.EDO
}

func X() No {
	return No{}
}

func main() {
	X()
}
