// Copyright 2018 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package p

import "io"

type A interface {
        io.Reader
}

func f(a A) {
        a.Read(nil)
}
