// rundir

// Copyright 2013 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// Issue 5105: linker segfaults on duplicate definition
// of a type..hash.* function.

package ignored
