// Code generated by go tool dist; DO NOT EDIT.

package main

const defaultPkgConfig = `pkg-config`
func defaultCC(goos, goarch string) string {
	switch goos+`/`+goarch {
	}
	return "/dev/null"
}
func defaultCXX(goos, goarch string) string {
	switch goos+`/`+goarch {
	}
	return "/dev/null"
}
