// compile

// Copyright 2009 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package main

func main() {
	x := 0;

	// this compiles
	switch x {
	case 0:
	}

	// this doesn't but should
	switch 0 {
	case 0:
	}
}


/*
bug158.go:14: fatal error: dowidth: unknown type: E-33
*/
